using System.Collections.Generic;
using AStarLogic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;

namespace AStar
{
    public class AStarExample : Game
    {
        GraphicsDeviceManager _graphics;
        SpriteBatch _spriteBatch;
        Texture2D _tilesTexture;
        SpriteFont _smallFont, _mediumFont;
        Map _map;
        MouseState _previousMouseState;
        KeyboardState _previousKeyState;
        double _startTime;

        public AStarExample()
        {
            _graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            int[,] karta =
            {
                {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1},
                {1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1},
                {1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1},
                {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}
            };
            _map = new Map(karta);
            this.IsMouseVisible = true;
            base.Initialize();
        }

        protected override void LoadContent()
        {
            _spriteBatch = new SpriteBatch(GraphicsDevice);
            _tilesTexture = Content.Load<Texture2D>("tiles");
            _smallFont = Content.Load<SpriteFont>("smallFont");
            _mediumFont = Content.Load<SpriteFont>("mediumFont");
        }

        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            var mouseState = Mouse.GetState();
            var keyState = Keyboard.GetState();

            if (mouseState.LeftButton == ButtonState.Pressed)
            {
                _map.SetWall(mouseState.X/32, mouseState.Y/32);
            }
            if (mouseState.RightButton == ButtonState.Pressed)
            {
                _map.UnSetWall(mouseState.X / 32, mouseState.Y / 32);
            }
            if (_previousKeyState.IsKeyUp(Keys.Q) && keyState.IsKeyDown(Keys.Q))
            {
                _map.SetStart(mouseState.X / 32, mouseState.Y / 32);
            }
            if (_previousKeyState.IsKeyUp(Keys.W) && keyState.IsKeyDown(Keys.W))
            {
                _map.SetStop(mouseState.X / 32, mouseState.Y / 32);
            }
            if (_previousKeyState.IsKeyUp(Keys.Enter) && keyState.IsKeyDown(Keys.Enter))
            {
                _map.MoveFromTo();
                _map.PlotRoute();
                _startTime = gameTime.TotalGameTime.TotalMilliseconds;
            }

            _previousMouseState = mouseState;
            _previousKeyState = keyState;

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            _spriteBatch.Begin();

            for (int y = 0; y < _map.SizeY; y++)
            {
                for (int x = 0; x < _map.SizeX; x++)
                {
                    var node = _map.MapData[y, x];
                    int tile = (int)node.Type;
                    if (tile > 0)
                    {
                        Rectangle sourceRectangle = new Rectangle((tile - 1)*32, 0, 32, 32);
                        _spriteBatch.Draw(_tilesTexture, new Vector2(x*32, y*32), sourceRectangle, Color.White);
                    }
                    if (node.IsVisited && gameTime.TotalGameTime.TotalMilliseconds > (_startTime + node.Time*50))
                    {
                        int dx = node.X - node.Sx;
                        int dy = node.Y - node.Sy;

                        Rectangle r = new Rectangle(32 + dx * 32, 64 + dy * 32, 32, 32);
                        _spriteBatch.Draw(_tilesTexture, new Vector2(node.X * 32, node.Y * 32), r, Color.White);

                        _spriteBatch.DrawString(_smallFont, node.F.ToString(), new Vector2(node.X * 32, node.Y * 32), Color.White);
                        _spriteBatch.DrawString(_smallFont, node.G.ToString(), new Vector2(node.X * 32, node.Y * 32 + 10), Color.White);
                        _spriteBatch.DrawString(_smallFont, node.H.ToString(), new Vector2(node.X * 32, node.Y * 32 + 20), Color.White);
                    }
                }
            }

            _spriteBatch.DrawString(_mediumFont, "press Q to set start and E to set goal, then ENTER to plot", new Vector2(10, _map.SizeY * 32 - 36), Color.White);
            _spriteBatch.DrawString(_mediumFont, "use left and right mouse button to edit map", new Vector2(10, _map.SizeY * 32 - 20), Color.White);

            _spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
